@@ -3,9 +3,10 @@
 #include <bytecode.h>
 
 #include <lttoolbox/compression.h>
-#include <apertium/string_utils.h>
-#include <apertium_re.h>
-#include <apertium/utf_converter.h>
+#include <lttoolbox/string_utils.h>
+#include <lttoolbox/input_file.h>
+#include <apertium/transfer_regex.h>
+#include <apertium/apertium_re.h>
 
 #include <iostream>
 #include <fstream>
@@ -14,28 +15,26 @@ using namespace std;
 
 PatternBuilder::PatternBuilder()
 {
-  alphabet.includeSymbol(L"<ANY_TAG>");
-  alphabet.includeSymbol(L"<ANY_CHAR>");
-  alphabet.includeSymbol(L"<LOOK:AHEAD>");
-  attr_items[L"lem"] = L"^(([^<]|\"\\<\")+)";
-  attr_items[L"lemq"] = L"\\#[- _][^<]+";
-  attr_items[L"lemh"] = L"^(([^<#]|\"\\<\"|\"\\#\")+)";
-  attr_items[L"whole"] = L"(.+)";
-  attr_items[L"tags"] = L"((<[^>]+>)+)";
-  attr_items[L"chname"] = L"({([^/]+)\\/)"; // includes delimiters { and / !!!
-  attr_items[L"chcontent"] = L"(\\{.+)";
-  attr_items[L"content"] = L"(\\{.+)";
-  attr_items[L"pos_tag"] = L"(<[^>]+>)";
-  starCanBeEmpty = false;
-  chunkVarCount = 0;
+  alphabet.includeSymbol("<ANY_TAG>"_u);
+  alphabet.includeSymbol("<ANY_CHAR>"_u);
+  alphabet.includeSymbol("<LOOK:AHEAD>"_u);
+  attr_items["lem"_u] = "^(([^<]|\"\\<\")+)"_u;
+  attr_items["lemq"_u] = "\\#[- _][^<]+"_u;
+  attr_items["lemh"_u] = "^(([^<#]|\"\\<\"|\"\\#\")+)"_u;
+  attr_items["whole"_u] = "(.+)"_u;
+  attr_items["tags"_u] = "((<[^>]+>)+)"_u;
+  attr_items["chname"_u] = "(\\{([^/]+)\\/)"_u; // includes delimiters { and / !!!
+  attr_items["chcontent"_u] = "(\\{.+)"_u;
+  attr_items["content"_u] = "(\\{.+)"_u;
+  attr_items["pos_tag"_u] = "(<[^>]+>)"_u;
 }
 
 int
-PatternBuilder::insertLemma(int const base, wstring const &lemma)
+PatternBuilder::insertLemma(int const base, UString const &lemma)
 {
   int retval = base;
-  static int const any_char = alphabet(L"<ANY_CHAR>");
-  if(lemma == L"")
+  static int const any_char = alphabet("<ANY_CHAR>"_u);
+  if(lemma.empty())
   {
     retval = transducer.insertSingleTransduction(any_char, retval);
     transducer.linkStates(retval, retval, any_char);
@@ -44,14 +43,14 @@ PatternBuilder::insertLemma(int const base, wstring const &lemma)
   {
     for(unsigned int i = 0, limit = lemma.size();  i != limit; i++)
     {
-      if(lemma[i] == L'\\')
+      if(lemma[i] == '\\')
       {
-        //retval = transducer.insertSingleTransduction(L'\\', retval);
+        //retval = transducer.insertSingleTransduction('\\', retval);
         i++;
         retval = transducer.insertSingleTransduction(int(lemma[i]),
                                                              retval);
       }
-      else if(lemma[i] == L'*')
+      else if(lemma[i] == '*')
       {
         retval = transducer.insertSingleTransduction(any_char, retval);
         transducer.linkStates(retval, retval, any_char);
@@ -68,13 +67,13 @@ PatternBuilder::insertLemma(int const base, wstring const &lemma)
 }
 
 int
-PatternBuilder::insertTags(int const base, const vector<wstring>& tags)
+PatternBuilder::insertTags(int const base, const vector<UString>& tags)
 {
   int retval = base;
-  static int const any_tag = alphabet(L"<ANY_TAG>");
+  static int const any_tag = alphabet("<ANY_TAG>"_u);
   for(unsigned int i = 0; i < tags.size(); i++)
   {
-    if(tags[i] == L"*")
+    if(tags[i] == "*"_u)
     {
       if(!starCanBeEmpty)
       {
@@ -84,10 +83,10 @@ PatternBuilder::insertTags(int const base, const vector<wstring>& tags)
     }
     else
     {
-      vector<wstring> tgs = StringUtils::split_wstring(tags[i], L".");
+      vector<UString> tgs = StringUtils::split(tags[i], "."_u);
       for(auto t : tgs)
       {
-        wstring tg = L"<" + t + L">";
+        UString tg = "<"_u + t + ">"_u;
         alphabet.includeSymbol(tg);
         retval = transducer.insertSingleTransduction(alphabet(tg), retval);
       }
@@ -99,105 +98,15 @@ PatternBuilder::insertTags(int const base, const vector<wstring>& tags)
 int
 PatternBuilder::countToFinalSymbol(const int count)
 {
-  const wstring count_sym = L"<RULE_NUMBER:" + to_wstring(count) + L">";
+  const UString count_sym = "<RULE_NUMBER:"_u + StringUtils::itoa(count) + ">"_u;
   alphabet.includeSymbol(count_sym);
   const int symbol = alphabet(count_sym);
   if(count != -1) final_symbols.insert(symbol);
   return symbol;
 }
 
-vector<PatternBuilder::TrieNode*>
-PatternBuilder::buildTrie(vector<wstring> parts)
-{
-  vector<TrieNode*> ret;
-  vector<vector<wstring>> p2;
-  for(auto p : parts)
-  {
-    if(p.size() == 0) continue;
-    bool found = false;
-    for(unsigned int t = 0; t < p2.size(); t++)
-    {
-      if(ret[t]->self == p[0])
-      {
-        p2[t].push_back(p.substr(1));
-        found = true;
-        break;
-      }
-    }
-    if(!found)
-    {
-      TrieNode* t = new TrieNode;
-      t->self = p[0];
-      ret.push_back(t);
-      p2.push_back(vector<wstring>(1, p.substr(1)));
-    }
-  }
-  for(unsigned int i = 0; i < ret.size(); i++)
-  {
-    ret[i]->next = buildTrie(p2[i]);
-  }
-  return ret;
-}
-
-wstring
-PatternBuilder::unbuildTrie(PatternBuilder::TrieNode* t)
-{
-  if(t->self == L'\0') return L"";
-  wstring single;
-  bool end = false;
-  vector<wstring> groups;
-  int ct = t->next.size();
-  for(auto it : t->next)
-  {
-    wstring blob = unbuildTrie(it);
-    if(blob.size() == 0)
-    {
-      end = true;
-      ct--;
-    }
-    else if(blob.size() == 1)
-    {
-      if(single.size() > 0) ct--;
-      single += blob;
-    }
-    else groups.push_back(blob);
-  }
-  wstring ret;
-  ret += t->self;
-  if(single.size() == 0 && groups.size() == 0) return ret;
-  if(single.size() > 1) single = L"[" + single + L"]";
-  if(ct > 1 || (groups.size() == 1 && end)) ret += L"(?:";
-  for(unsigned int i = 0; i < groups.size(); i++)
-  {
-    if(i > 0) ret += L"|";
-    ret += groups[i];
-  }
-  if(single.size() > 0)
-  {
-    if(groups.size() > 0) ret += L"|";
-    ret += single;
-  }
-  if(ct > 1 || (groups.size() == 1 && end)) ret += L")";
-  if(end) ret += L"?";
-  return ret;
-}
-
-wstring
-PatternBuilder::trie(vector<wstring> parts)
-{
-  if(parts.size() == 0) return L"";
-  for(unsigned int i = 0; i < parts.size(); i++)
-  {
-    parts[i] = L"<" + parts[i];
-    parts[i] += L'\0';
-  }
-  vector<TrieNode*> l = buildTrie(parts);
-  // they all start with L'<', so there will only be 1.
-  return L"(" + unbuildTrie(l[0]) + L">)";
-}
-
 void
-PatternBuilder::addPattern(vector<vector<PatternElement*>> pat, int rule, double weight, bool isLex)
+PatternBuilder::addPattern(const vector<vector<PatternElement*>>& pat, int rule, double weight, bool isLex)
 {
   int state = transducer.getInitial();
   for(unsigned int p = 0; p < pat.size(); p++)
@@ -213,17 +122,17 @@ PatternBuilder::addPattern(vector<vector<PatternElement*>> pat, int rule, double
           if(pe->tags.size() > 0) lookahead[state].push_back(pe->tags[0]);
         }
       }
-      state = transducer.insertSingleTransduction(L' ', state);
+      state = transducer.insertSingleTransduction(' ', state);
     }
-    state = transducer.insertNewSingleTransduction(L'^', state);
+    state = transducer.insertNewSingleTransduction('^', state);
     int end = insertLemma(state, pat[p][0]->lemma);
     end = insertTags(end, pat[p][0]->tags);
-    end = transducer.insertSingleTransduction(L'$', end);
+    end = transducer.insertSingleTransduction('$', end);
     for(unsigned int i = 1; i < pat[p].size(); i++)
     {
       int temp = insertLemma(state, pat[p][i]->lemma);
       temp = insertTags(temp, pat[p][i]->tags);
-      transducer.linkStates(temp, end, L'$');
+      transducer.linkStates(temp, end, '$');
     }
     state = end;
   }
@@ -233,7 +142,7 @@ PatternBuilder::addPattern(vector<vector<PatternElement*>> pat, int rule, double
 }
 
 void
-PatternBuilder::addRule(int rule, double weight, vector<vector<PatternElement*>> pattern, vector<wstring> firstChunk, wstring name)
+PatternBuilder::addRule(int rule, double weight, const vector<vector<PatternElement*>>& pattern, const vector<UString>& firstChunk, const UString& name)
 {
   rules[rule] = make_pair(firstChunk, pattern);
   addPattern(pattern, rule, weight, false);
@@ -254,82 +163,67 @@ PatternBuilder::addRule(int rule, double weight, vector<vector<PatternElement*>>
 }
 
 void
-PatternBuilder::addList(wstring name, set<wstring, Ltstr> vals)
+PatternBuilder::addList(const UString& name, const set<UString>& vals)
 {
   lists[name] = vals;
 }
 
 void
-PatternBuilder::addAttr(wstring name, set<wstring, Ltstr> vals)
+PatternBuilder::addAttr(const UString& name, const set<UString>& vals)
 {
-  /*wstring pat = L"(";
-  for(set<wstring, Ltstr>::iterator it = vals.begin(); it != vals.end(); it++)
-  {
-    if(pat.size() > 1)
-    {
-      pat += L"|";
-    }
-    pat += L"<" + StringUtils::substitute(*it, L".", L"><") + L">";
-  }
-  pat += L")";
-  attr_items[name] = pat;*/
-  vector<wstring> pat;
-  for(auto it : vals)
-  {
-    wstring p = StringUtils::substitute(it, L"\\.", L"<>");
-    p = StringUtils::substitute(p, L".", L"><");
-    pat.push_back(StringUtils::substitute(p, L"<>", L"\\."));
-  }
-  wstring pt = trie(pat);
-  //wcerr << name << "\t" << pt << endl;
-  attr_items[name] = pt;
+  vector<UString> pat;
+  pat.assign(vals.begin(), vals.end());
+  attr_items[name] = optimize_regex(pat);
+  // compile now so we can catch errors at build time
+  ApertiumRE r;
+  r.compile(attr_items[name]);
 }
 
 bool
-PatternBuilder::isAttrDefined(wstring name)
+PatternBuilder::isAttrDefined(const UString& name)
 {
   return attr_items.find(name) != attr_items.end();
 }
 
 void
-PatternBuilder::addVar(wstring name, wstring val)
+PatternBuilder::addVar(const UString& name, const UString& val)
 {
   variables[name] = val;
 }
 
-wstring
-PatternBuilder::BCstring(const wstring& s)
+UString
+PatternBuilder::BCstring(const UString& s)
 {
-  wstring ret;
+  UString ret;
   ret += STRING;
-  ret += (wchar_t)s.size();
+  ret += (UChar)s.size();
   ret += s;
   return ret;
 }
 
-wstring
-PatternBuilder::BCifthenelse(const wstring& cond, const wstring& yes, const wstring& no)
+UString
+PatternBuilder::BCifthenelse(const UString& cond, const UString& yes, const UString& no)
 {
-  wstring ret = cond;
+  UString ret = cond;
   if(yes.size() == 0)
   {
     ret += JUMPONTRUE;
-    ret += (wchar_t)no.size();
+    ret += (UChar)no.size();
     ret += no;
   }
   else if(no.size() == 0)
   {
     ret += JUMPONFALSE;
-    ret += (wchar_t)yes.size();
+    ret += (UChar)yes.size();
     ret += yes;
   }
   else
   {
     ret += JUMPONFALSE;
-    ret += (wchar_t)(yes.size() + 2);
+    ret += (UChar)(yes.size() + 2);
     ret += yes;
     ret += JUMP;
-    ret += (wchar_t)no.size();
+    ret += (UChar)no.size();
     ret += no;
   }
   return ret;
@@ -341,11 +235,11 @@ PatternBuilder::buildLookahead()
   for(auto it : firstSet)
   {
     firstSet[it.first].insert(it.first);
-    vector<wstring> todo;
+    vector<UString> todo;
     for(auto op : it.second) todo.push_back(op);
     while(todo.size() > 0)
     {
-      wstring cur = todo.back();
+      UString cur = todo.back();
       todo.pop_back();
       if(cur != it.first && firstSet.find(cur) != firstSet.end())
       {
@@ -362,9 +256,9 @@ PatternBuilder::buildLookahead()
   }
   for(auto it : lookahead)
   {
-    int state = transducer.insertSingleTransduction(alphabet(L"<LOOK:AHEAD>"), it.first);
-    state = transducer.insertSingleTransduction(L'^', state);
-    transducer.linkStates(state, state, alphabet(L"<ANY_CHAR>"));
+    int state = transducer.insertSingleTransduction(alphabet("<LOOK:AHEAD>"_u), it.first);
+    state = transducer.insertSingleTransduction('^', state);
+    transducer.linkStates(state, state, alphabet("<ANY_CHAR>"_u));
     int end = -1;
     for(auto next : it.second)
     {
@@ -372,19 +266,19 @@ PatternBuilder::buildLookahead()
       for(auto tag : firstSet[next])
       {
         int temp = state;
-        if(tag != L"*")
+        if(tag != "*"_u)
         {
-          temp = transducer.insertSingleTransduction(alphabet(L"<" + tag + L">"), temp);
+          temp = transducer.insertSingleTransduction(alphabet("<"_u + tag + ">"_u), temp);
         }
-        transducer.linkStates(temp, temp, alphabet(L"<ANY_TAG>"));
+        transducer.linkStates(temp, temp, alphabet("<ANY_TAG>"_u));
         if(end == -1)
         {
-          end = transducer.insertSingleTransduction(L'$', temp);
+          end = transducer.insertSingleTransduction('$', temp);
           transducer.setFinal(end);
         }
         else
         {
-          transducer.linkStates(temp, end, L'$');
+          transducer.linkStates(temp, end, '$');
         }
       }
     }
@@ -401,7 +295,7 @@ PatternBuilder::isPrefix(const vector<vector<PatternElement*>>& rule, const vect
     for(auto r : rule[i])
     {
       if(r->tags.size() == 0) continue;
-      else if(r->tags[0] == L"*")
+      else if(r->tags[0] == "*"_u)
       {
         found = true;
         break;
@@ -409,7 +303,7 @@ PatternBuilder::isPrefix(const vector<vector<PatternElement*>>& rule, const vect
       for(auto p : prefix[i])
       {
         if(p->tags.size() == 0) continue;
-        else if(p->tags[0] == L"*" || p->tags[0] == r->tags[0])
+        else if(p->tags[0] == "*"_u || p->tags[0] == r->tags[0])
         {
           found = true;
           break;
@@ -429,7 +323,7 @@ PatternBuilder::buildFallback()
   starCanBeEmpty = true;
   vector<PatternElement*> fallback;
   PatternElement* fall = new PatternElement;
-  fall->tags.push_back(L"FALL:BACK");
+  fall->tags.push_back("FALL:BACK"_u);
   fallback.push_back(fall);
   for(auto rule : rules)
   {
@@ -438,13 +332,13 @@ PatternBuilder::buildFallback()
     {
       PatternElement* pe = new PatternElement;
       pe->tags.push_back(tg);
-      pe->tags.push_back(L"*");
+      pe->tags.push_back("*"_u);
       result.push_back(pe);
     }
     vector<vector<PatternElement*>> resultPat;
     resultPat.push_back(result);
-    set<wstring> patPrefix;
-    set<wstring> resultPrefix;
+    set<UString> patPrefix;
+    set<UString> resultPrefix;
     for(auto rule2 : rules)
     {
       if(isPrefix(rule2.second.second, resultPat))
@@ -474,7 +368,7 @@ PatternBuilder::buildFallback()
       {
         PatternElement* pe = new PatternElement;
         pe->tags.push_back(it);
-        pe->tags.push_back(L"*");
+        pe->tags.push_back("*"_u);
         add.push_back(pe);
       }
       resultPat.push_back(add);
@@ -491,33 +385,24 @@ PatternBuilder::buildFallback()
 void
 PatternBuilder::loadLexFile(const string& fname)
 {
-  wifstream lex;
-  lex.open(fname);
-  if(!lex.is_open())
-  {
-    wcerr << "Unable to open file " << fname.c_str() << " for reading." << endl;
-    exit(EXIT_FAILURE);
-  }
-  while(!lex.eof())
-  {
-    wstring name;
-    while(!lex.eof() && lex.peek() != L'\t') name += lex.get();
+  InputFile lex;
+  lex.open_or_exit(fname.c_str());
+  while(!lex.eof()) {
+    UString name;
+    while(!lex.eof() && lex.peek() != '\t') name += lex.get();
     lex.get();
-    wstring weight;
-    while(!lex.eof() && lex.peek() != L'\t') weight += lex.get();
+    UString weight;
+    while(!lex.eof() && lex.peek() != '\t') weight += lex.get();
     lex.get();
     if(lex.eof()) break;
     vector<vector<PatternElement*>> pat;
-    while(!lex.eof() && lex.peek() != L'\n')
-    {
+    while(!lex.eof() && lex.peek() != '\n') {
       PatternElement* p = new PatternElement;
-      while(lex.peek() != L'@') p->lemma += lex.get();
+      while(lex.peek() != '@') p->lemma += u_tolower(lex.get());
       lex.get();
-      wstring tag;
-      while(lex.peek() != L' ' && lex.peek() != L'\n')
-      {
-        if(lex.peek() == L'.')
-        {
+      UString tag;
+      while(lex.peek() != ' ' && lex.peek() != '\n') {
+        if(lex.peek() == '.') {
           lex.get();
           p->tags.push_back(tag);
           tag.clear();
@@ -525,29 +410,29 @@ PatternBuilder::loadLexFile(const string& fname)
         else tag += lex.get();
       }
       p->tags.push_back(tag);
-      if(lex.peek() == L' ') lex.get();
+      if(lex.peek() == ' ') lex.get();
       pat.push_back(vector<PatternElement*>(1, p));
     }
     lex.get();
-    lexicalizations[name].push_back(make_pair(stod(weight), pat));
+    lexicalizations[name].push_back(make_pair(StringUtils::stod(weight), pat));
   }
 }
 
 void
-PatternBuilder::write(FILE* output, int longest, vector<pair<int, wstring>> inputBytecode, vector<wstring> outputBytecode)
+PatternBuilder::write(FILE* output, int longest, vector<pair<int, UString>> inputBytecode, vector<UString> outputBytecode)
 {
-  Compression::multibyte_write(longest, output);
+  Compression::multibyte_write((longest > 0 ? longest : 0), output);
   Compression::multibyte_write(inputBytecode.size(), output);
   for(unsigned int i = 0; i < inputBytecode.size(); i++)
   {
     Compression::multibyte_write(inputBytecode[i].first, output);
-    Compression::wstring_write(inputBytecode[i].second, output);
+    Compression::string_write(inputBytecode[i].second, output);
   }
 
   Compression::multibyte_write(outputBytecode.size(), output);
   for(unsigned int i = 0; i < outputBytecode.size(); i++)
   {
-    Compression::wstring_write(outputBytecode[i], output);
+    Compression::string_write(outputBytecode[i], output);
   }
 
   Compression::multibyte_write(chunkVarCount, output);
@@ -564,7 +449,7 @@ PatternBuilder::write(FILE* output, int longest, vector<pair<int, wstring>> inpu
   // Find all arcs with "final_symbols" in the transitions, let their source node instead be final,
   // and extract the rule number from the arc. Record relation between source node and rule number
   // in finals_rules. It is now no longer safe to minimize -- but we already did that.
-  const wstring rule_sym_pre = L"<RULE_NUMBER:"; // see countToFinalSymbol()
+  const UString rule_sym_pre = "<RULE_NUMBER:"_u; // see countToFinalSymbol()
   for(map<int, multimap<int, pair<int, double> > >::const_iterator it = transitions.begin(),
         limit = transitions.end(); it != limit; ++it)
   {
@@ -578,12 +463,12 @@ PatternBuilder::write(FILE* output, int longest, vector<pair<int, wstring>> inpu
         continue;
       }
       // Extract the rule number encoded by countToFinalSymbol():
-      wstring s;
+      UString s;
       alphabet.getSymbol(s, symbol);
       if(s.compare(0, rule_sym_pre.size(), rule_sym_pre) != 0) {
         continue;
       }
-      const int rule_num = stoi(s.substr(rule_sym_pre.size()));
+      const int rule_num = StringUtils::stoi(s.substr(rule_sym_pre.size()));
       transducer.setFinal(src);
       finals_rules.insert(make_pair(src, make_pair(rule_num, wgt)));
     }
@@ -609,54 +494,42 @@ PatternBuilder::write(FILE* output, int longest, vector<pair<int, wstring>> inpu
 
   // attr_items
 
-  // precompiled regexps
-  Compression::string_write(string(pcre_version()), output);
+  // empty version number since we're not on PCRE anymore
+  Compression::multibyte_write(0, output);
   Compression::multibyte_write(attr_items.size(), output);
 
-  map<wstring, wstring, Ltstr>::iterator it, limit;
-  for(it = attr_items.begin(), limit = attr_items.end(); it != limit; it++)
-  {
-    Compression::wstring_write(it->first, output);
-    ApertiumRE my_re;
-    my_re.compile(UtfConverter::toUtf8(it->second));
-    my_re.write(output);
-    Compression::wstring_write(it->second, output);
+  for (auto& it : attr_items) {
+    Compression::string_write(it.first, output);
+    Compression::multibyte_write(0, output); // empty binary form of regex
+    Compression::string_write(it.second, output);
   }
 
   // variables
   Compression::multibyte_write(variables.size(), output);
-  for(map<wstring, wstring, Ltstr>::const_iterator it = variables.begin(), limit = variables.end();
-      it != limit; it++)
-  {
-    Compression::wstring_write(it->first, output);
-    Compression::wstring_write(it->second, output);
+  for (auto& it : variables) {
+    Compression::string_write(it.first, output);
+    Compression::string_write(it.second, output);
   }
 
   // lists
   Compression::multibyte_write(lists.size(), output);
-  for(map<wstring, set<wstring, Ltstr>, Ltstr>::const_iterator it = lists.begin(), limit = lists.end();
-      it != limit; it++)
-  {
-    Compression::wstring_write(it->first, output);
-    Compression::multibyte_write(it->second.size(), output);
+  for (auto& it : lists) {
+    Compression::string_write(it.first, output);
+    Compression::multibyte_write(it.second.size(), output);
 
-    for(set<wstring, Ltstr>::const_iterator it2 = it->second.begin(), limit2 = it->second.end();
-  it2 != limit2; it2++)
-    {
-      Compression::wstring_write(*it2, output);
+    for (auto& it2 : it.second) {
+      Compression::string_write(it2, output);
     }
   }
 
   // rule names
   Compression::multibyte_write(inRuleNames.size(), output);
-  for(unsigned int i = 0; i < inRuleNames.size(); i++)
-  {
-    Compression::wstring_write(inRuleNames[i], output);
+  for (auto& name : inRuleNames) {
+    Compression::string_write(name, output);
   }
   Compression::multibyte_write(outRuleNames.size(), output);
-  for(unsigned int i = 0; i < outRuleNames.size(); i++)
-  {
-    Compression::wstring_write(outRuleNames[i], output);
+  for (auto& name : outRuleNames) {
+    Compression::string_write(name, output);
   }
 
 }
