@@ -2,16 +2,13 @@
 #define __RTXPROCESSOR__
 
 #include <rtx_config.h>
-#include <apertium_re.h>
-#include <apertium/utf_converter.h>
+#include <apertium/apertium_re.h>
 #include <lttoolbox/alphabet.h>
-#include <lttoolbox/buffer.h>
-#include <lttoolbox/ltstr.h>
 #include <matcher.h>
 #include <chunk.h>
 #include <pool.h>
+#include <lttoolbox/input_file.h>
 
-#include <cstring>
 #include <cstdio>
 #include <map>
 #include <set>
@@ -24,7 +21,7 @@ struct StackElement
   int mode;
   bool b;
   int i;
-  wstring s;
+  UString s;
   Chunk* c;
 };
 
@@ -49,52 +46,52 @@ private:
   /**
    * Attribute category regular expressions
    */
-  map<wstring, ApertiumRE, Ltstr> attr_items;
+  map<UString, ApertiumRE> attr_items;
 
   /**
    * Virtual machine global variables
    * name => value
    */
-  map<wstring, wstring, Ltstr> variables;
+  map<UString, UString> variables;
   
   /**
    * Virtual machine global variables to wblank map
    * name => value
    */
-  map<wstring, wstring, Ltstr> wblank_variables;
+  map<UString, UString> wblank_variables;
 
   /**
    * Lists
    * name => { values }
    */
-  map<wstring, set<wstring, Ltstr>, Ltstr> lists;
+  map<UString, set<UString>> lists;
 
   /**
    * Lists, but all values are converted to lower case
    * Used for case-insensitive comparison
    * name => { values }
    */
-  map<wstring, set<wstring, Ltstr>, Ltstr> listslow;
+  map<UString, set<UString>> listslow;
 
   /**
    * Bytecode for input-time rules
    */
-  vector<wstring> rule_map;
+  vector<UString> rule_map;
 
   /**
    * Bytecode for output-time rules
    */
-  vector<wstring> output_rules;
+  vector<UString> output_rules;
 
   /**
    * Debug names for input-time rules (may be empty)
    */
-  vector<wstring> inRuleNames;
+  vector<UString> inRuleNames;
 
   /**
    * Debug names for output-time rules (may be empty)
    */
-  vector<wstring> outRuleNames;
+  vector<UString> outRuleNames;
 
   /**
    * Length of pattern of each input-time rule, including blanks
@@ -114,7 +111,7 @@ private:
   /**
    * false if EOF or \0 has been reached in the input stream, true otherwise
    */
-  bool furtherInput;
+  bool furtherInput = true;
 
   /**
    * The stack used by the virtual machine
@@ -133,12 +130,12 @@ private:
    * A parallel stack to store wordbound blanks that mimics the operations
    * of the main stack. wblanks are added everytime lemmas are clipped
    */
-  wstring theWblankStack[32];
+  UString theWblankStack[32];
   
   /**
    * wordbound blank to be output
    */
-  wstring out_wblank;
+  UString out_wblank;
 
   /**
    * Input to the virtual machine
@@ -173,7 +170,7 @@ private:
    * then we want to output it directly, particularly if it's empty
    * and because of lookahead, only processGLR() knows which blanks are which
    */
-  list<wstring> blankQueue;
+  list<UString> blankQueue;
 
   /**
    * The parse stack
@@ -211,7 +208,7 @@ private:
    * Branch of parseGraph currently being operated on
    * Needed by applyRule() for FETCHCHUNK and SETCHUNK
    */
-  ParseNode* currentBranch;
+  ParseNode* currentBranch = nullptr;
 
   //////////
   // SETTINGS
@@ -219,122 +216,95 @@ private:
 
   /**
    * true if the next input token should be parsed as an LU, false otherwise
-   * Initial value: false
    */
-  bool inword;
+  bool inword = false;
   
   /**
    * true if the next input token should be parsed as a wordbound blank, false otherwise
-   * Initial value: false
    */
-  bool inwblank;
+  bool inwblank = false;
 
   /**
    * Whether output should flush on \0
-   * Default: false
    */
-  bool null_flush;
+  bool null_flush = false;
 
   /**
-   * If true, each instruction of virtual machine will be printed to wcerr
-   * Default: false
+   * If true, each instruction of virtual machine will be printed to cerr
    */
-  bool printingSteps;
+  bool printingSteps = false;
 
   /**
-   * If true, each rule that is applied will be printed to wcerr
-   * Default: false
+   * If true, each rule that is applied will be printed to cerr
    */
-  bool printingRules;
+  bool printingRules = false;
 
   /**
-   * If true, each action of filterParseGraph() will be logged to wcerr
-   * Default: false
+   * If true, each action of filterParseGraph() will be logged to cerr
    */
-  bool printingBranches;
+  bool printingBranches = false;
 
   /**
    * If true, produce a full report, similar to (printingRules && printingBranches)
    * Affected by treePrintMode
-   * Default: false
    */
-  bool printingAll;
+  bool printingAll = false;
 
   /**
    * false if input comes from apertium-anaphora, true otherwise
-   * Default: true
    */
-  bool noCoref;
+  bool noCoref = true;
 
   /**
    * true if rule application should mimic the chunker-interchunk-postchunk
    * pipeline, false otherwise
-   * Default: false
    */
-  bool isLinear;
+  bool isLinear = false;
 
   /**
    * If true, parse tree will be printed according to treePrintMode
    * before output-time rules are applied
-   * Default: false
    */
-  bool printingTrees;
+  bool printingTrees = false;
 
   /**
    * If false, output-time rules will not be applied and linear output
    * will not be produced
-   * Default: true
    */
-  bool printingText;
+  bool printingText = true;
 
   /**
    * Manner in which to print trees
    * Set by setOutputMode()
    * Enum defined in chunk.h
-   * Default: TreeModeNest
    */
-  TreeMode treePrintMode;
+  TreeMode treePrintMode = TreeModeNest;
 
   /**
    * Counter used to give distinct, consistent identifiers to ParseNodes
    * for tracing purposes
    */
-  int newBranchId;
+  int newBranchId = 0;
 
   /**
    * If this is set to true, filterParseGraph() will only discard branches
    * on parse error
    */
-  bool noFilter;
+  bool noFilter = true;
 
   //////////
   // VIRTUAL MACHINE
   //////////
 
-  /**
-   * Determine capitalization of a string
-   * @param str - input string
-   * @return L"AA", L"Aa", or L"aa"
-   */
-  wstring caseOf(wstring const &str);
-
-  /**
-   * Produce a version of target_word with the case of source_word
-   * @param source_word - source of case
-   * @param target_word - source of content
-   * @return generated string
-   */
-  wstring copycase(wstring const &source_word, wstring const &target_word);
-
   /**
    * Return whether str1 begins with str2
    */
-  bool beginsWith(wstring const &str1, wstring const &str2) const;
+  bool beginsWith(UString const &str1, UString const &str2) const;
 
   /**
    * Return whether str1 ends with str2
    */
-  bool endsWith(wstring const &str1, wstring const &str2) const;
+  bool endsWith(UString const &str1, UString const &str2) const;
 
   /**
    * The virtual machine
@@ -343,7 +313,7 @@ private:
    * @param rule - bytecode for rule to be applied
    * @return false if REJECTRULE was executed, true otherwise
    */
-  bool applyRule(const wstring& rule);
+  bool applyRule(const UString& rule);
 
   /**
    * Pop and return a boolean from theStack
@@ -358,20 +328,20 @@ private:
   int popInt();
 
   /**
-   * Pop and return a wstring from theStack
-   * Log error and call exit(1) if top element is not a wstring
+   * Pop and return a UString from theStack
+   * Log error and call exit(1) if top element is not a UString
    */
-  wstring popString();
+  UString popString();
 
   /**
    * Equivalent to popString(), but with called as
-   * wstring x; popString(x);
+   * UString x; popString(x);
    * rather than
-   * wstring x = popString();
+   * UString x = popString();
    * This uses a swap to save an allocation and a copy, which is almost twice
    * as fast, which has a noticeable impact on overall speed
    */
-  void popString(wstring& dest);
+  void popString(UString& dest);
 
   /**
    * Pop and return a Chunk pointer from theStack
@@ -391,7 +361,7 @@ private:
     theStack[stackIdx].i = i;
     theWblankStack[stackIdx].clear();
   }
-  inline void pushStack(const wstring& s, wstring wbl = L"")
+  inline void pushStack(const UString& s, UString wbl = ""_u)
   {
     theStack[++stackIdx].mode = 2;
     theStack[stackIdx].s.assign(s);
@@ -413,13 +383,15 @@ private:
   // RULE SELECTION AND I/O
   //////////
 
+  InputFile infile;
+
   /**
    * Read an LU or a blank
    * Modifies: furtherInput
    * @param in - input stream
    * @return pointer to token read
    */
-  Chunk* readToken(FILE *in);
+  Chunk* readToken();
 
   bool lookahead(ParseNode* node);
 
@@ -434,13 +406,13 @@ private:
   /**
    * Output the next blank in blankQueue, or a space if the queue is empty
    */
-  void writeBlank(FILE* out);
+  void writeBlank(UFILE* out);
 
   /**
    * Apply output-time rules and write nodes to output stream
    * @param out - output stream
    */
-  void outputAll(FILE* out);
+  void outputAll(UFILE* out);
 
   /**
    * Prune any ParseNodes that have reached error states
@@ -453,7 +425,7 @@ private:
    * Process input as a GLR parser
    * Read input, call checkForReduce(), call filterParseGraph(), call outputAll()
    */
-  void processGLR(FILE* in, FILE* out);
+  void processGLR(UFILE* out);
 
   /**
    * Apply longest rule matching the beginning of t1x and append the result to t2x
@@ -464,19 +436,19 @@ private:
    * Mimic apertium-transfer | apertium-interchunk | apertium-postchunk
    * Read input, call processTRXLayer twice, apply output-time rules, output
    */
-  void processTRX(FILE* in, FILE* out);
+  void processTRX(UFILE* out);
   
   /**
    * True if clipping lem/lemh/whole
   */
-  bool gettingLemmaFromWord(wstring attr);
+  bool gettingLemmaFromWord(UString attr);
   
 public:
   RTXProcessor();
   ~RTXProcessor();
 
   void read(string const &filename);
-  void process(FILE *in, FILE *out);
+  void process(FILE *in, UFILE *out);
   bool getNullFlush(void);
   void setNullFlush(bool null_flush);
   void printSteps(bool val)
